@@ -253,7 +253,7 @@ Run `pdoc --math`, and pdoc will render formulas in your docstrings. See
 
 ## ...add my project's logo?
 
-See [*Configuring pdoc*](#configuring-pdoc).
+See [*Customizing pdoc*](#customizing-pdoc).
 
 
 ## ...include Markdown files?
@@ -390,7 +390,7 @@ You can find an example in [`examples/library-usage`](https://github.com/mitmpro
 from __future__ import annotations
 
 __docformat__ = "markdown"  # explicitly disable rST processing in the examples above.
-__version__ = "11.0.0"  # this is read from setup.py
+__version__ = "12.0.0"  # this is read from setup.py
 
 import traceback
 import warnings
