@@ -33,7 +33,7 @@ from functools import wraps
 from pathlib import Path
 from typing import Any, ClassVar, Generic, TypeVar, Union
 
-from pdoc import doc_ast, extract
+from pdoc import doc_ast, doc_pyi, extract
 from pdoc.doc_types import (
     GenericAlias,
     NonUserDefinedCallables,
@@ -215,15 +215,15 @@ class Namespace(Doc[T], metaclass=ABCMeta):
     def _var_docstrings(self) -> dict[str, str]:
         """A mapping from some member variable names to their docstrings."""
 
-    @abstractmethod
-    def _taken_from(self, member_name: str, obj: Any) -> tuple[str, str]:
-        """The location this member was taken from. If unknown, (modulename, qualname) is returned."""
-
     @cached_property
     @abstractmethod
     def _var_annotations(self) -> dict[str, Any]:
         """A mapping from some member variable names to their type annotations."""
 
+    @abstractmethod
+    def _taken_from(self, member_name: str, obj: Any) -> tuple[str, str]:
+        """The location this member was taken from. If unknown, (modulename, qualname) is returned."""
+
     @cached_property
     @abstractmethod
     def own_members(self) -> list[Doc]:
@@ -304,6 +304,12 @@ class Namespace(Doc[T], metaclass=ABCMeta):
             if self._var_docstrings.get(name):
                 doc.docstring = self._var_docstrings[name]
             members[doc.name] = doc
+
+        if isinstance(self, Module):
+            # quirk: doc_pyi expects .members to be set already
+            self.members = members  # type: ignore
+            doc_pyi.include_typeinfo_from_stub_files(self)
+
         return members
 
     @cached_property
@@ -394,6 +400,14 @@ class Module(Namespace[types.ModuleType]):
     def _var_docstrings(self) -> dict[str, str]:
         return doc_ast.walk_tree(self.obj).docstrings
 
+    @cached_property
+    def _var_annotations(self) -> dict[str, Any]:
+        annotations = doc_ast.walk_tree(self.obj).annotations.copy()
+        for k, v in _safe_getattr(self.obj, "__annotations__", {}).items():
+            annotations[k] = v
+
+        return resolve_annotations(annotations, self.obj, self.fullname)
+
     def _taken_from(self, member_name: str, obj: Any) -> tuple[str, str]:
         if obj is empty:
             return self.modulename, f"{self.qualname}.{member_name}".lstrip(".")
@@ -410,14 +424,6 @@ class Module(Namespace[types.ModuleType]):
                 "."
             )
 
-    @cached_property
-    def _var_annotations(self) -> dict[str, Any]:
-        annotations = doc_ast.walk_tree(self.obj).annotations.copy()
-        for k, v in _safe_getattr(self.obj, "__annotations__", {}).items():
-            annotations[k] = v
-
-        return resolve_annotations(annotations, self.obj, self.fullname)
-
     @cached_property
     def own_members(self) -> list[Doc]:
         return list(self.members.values())
@@ -544,6 +550,15 @@ class Class(Namespace[type]):
     def __repr__(self):
         return f"<{_decorators(self)}class {self.modulename}.{self.qualname}{_docstr(self)}{_children(self)}>"
 
+    @cached_property
+    def docstring(self) -> str:
+        doc = Doc.docstring.__get__(self)  # type: ignore
+        if doc == dict.__doc__:
+            # Don't display default docstring for dict subclasses (primarily TypedDict).
+            return ""
+        else:
+            return doc
+
     @cached_property
     def _var_docstrings(self) -> dict[str, str]:
         docstrings: dict[str, str] = {}
@@ -552,31 +567,6 @@ class Class(Namespace[type]):
                 docstrings.setdefault(name, docstr)
         return docstrings
 
-    @cached_property
-    def _declarations(self) -> dict[str, tuple[str, str]]:
-        decls: dict[str, tuple[str, str]] = {}
-        for cls in self.obj.__mro__:
-            treeinfo = doc_ast.walk_tree(cls)
-            for name in treeinfo.docstrings.keys() | treeinfo.annotations.keys():
-                decls.setdefault(name, (cls.__module__, f"{cls.__qualname__}.{name}"))
-            for name in cls.__dict__:
-                decls.setdefault(name, (cls.__module__, f"{cls.__qualname__}.{name}"))
-        if decls.get("__init__", None) == ("builtins", "object.__init__"):
-            decls["__init__"] = (
-                self.obj.__module__,
-                f"{self.obj.__qualname__}.__init__",
-            )
-        return decls
-
-    def _taken_from(self, member_name: str, obj: Any) -> tuple[str, str]:
-        try:
-            return self._declarations[member_name]
-        except KeyError:  # pragma: no cover
-            warnings.warn(
-                f"Cannot determine where {self.fullname}.{member_name} is taken from, assuming current file."
-            )
-            return self.modulename, f"{self.qualname}.{member_name}"
-
     @cached_property
     def _var_annotations(self) -> dict[str, type]:
         # this is a bit tricky: __annotations__ also includes annotations from parent classes,
@@ -608,6 +598,32 @@ class Class(Namespace[type]):
 
         return {k: v[1] for k, v in annotations.items()}
 
+    @cached_property
+    def _declarations(self) -> dict[str, tuple[str, str]]:
+        decls: dict[str, tuple[str, str]] = {}
+        for cls in self.obj.__mro__:
+            treeinfo = doc_ast.walk_tree(cls)
+            for name in treeinfo.docstrings.keys() | treeinfo.annotations.keys():
+                decls.setdefault(name, (cls.__module__, f"{cls.__qualname__}.{name}"))
+            for name in cls.__dict__:
+                decls.setdefault(name, (cls.__module__, f"{cls.__qualname__}.{name}"))
+        if decls.get("__init__", None) == ("builtins", "object.__init__"):
+            decls["__init__"] = (
+                self.obj.__module__,
+                f"{self.obj.__qualname__}.__init__",
+            )
+        return decls
+
+    def _taken_from(self, member_name: str, obj: Any) -> tuple[str, str]:
+        try:
+            return self._declarations[member_name]
+        except KeyError:  # pragma: no cover
+            # TypedDict botches __mro__ and may need special casing here.
+            warnings.warn(
+                f"Cannot determine where {self.fullname}.{member_name} is taken from, assuming current file."
+            )
+            return self.modulename, f"{self.qualname}.{member_name}"
+
     @cached_property
     def own_members(self) -> list[Doc]:
         members = self._members_by_origin.get((self.modulename, self.qualname), [])
@@ -639,6 +655,9 @@ class Class(Namespace[type]):
                 # Special case: Do not show a constructor for enums. They are typically not constructed by users.
                 # The alternative would be showing __new__, as __call__ is too verbose.
                 del unsorted["__init__"]
+            elif issubclass(self.obj, dict):
+                # Special case: Do not show a constructor for dict subclasses.
+                del unsorted["__init__"]
             else:
                 # Check if there's a helpful Metaclass.__call__ or Class.__new__. This dance is very similar to
                 # https://github.com/python/cpython/blob/9feae41c4f04ca27fd2c865807a5caeb50bf4fc4/Lib/inspect.py#L2359-L2376
@@ -995,14 +1014,15 @@ class _PrettySignature(inspect.Signature):
     for complex signatures.
     """
 
-    def __str__(self):
+    MULTILINE_CUTOFF = 70
+
+    def _params(self) -> list[str]:
         # redeclared here to keep code snipped below as-is.
         _POSITIONAL_ONLY = inspect.Parameter.POSITIONAL_ONLY
         _VAR_POSITIONAL = inspect.Parameter.VAR_POSITIONAL
         _KEYWORD_ONLY = inspect.Parameter.KEYWORD_ONLY
-        _empty = empty
 
-        # https://github.com/python/cpython/blob/799f8489d418b7f9207d333eac38214931bd7dcc/Lib/inspect.py#L3083-L3123
+        # https://github.com/python/cpython/blob/799f8489d418b7f9207d333eac38214931bd7dcc/Lib/inspect.py#L3083-L3117
         # Change: added re.sub() to formatted = ....
         # ✂ start ✂
         result = []
@@ -1040,18 +1060,28 @@ class _PrettySignature(inspect.Signature):
             # There were only positional-only parameters, hence the
             # flag was not reset to 'False'
             result.append("/")
-
-        rendered = "({})".format(", ".join(result))
-
-        if self.return_annotation is not _empty:
-            anno = formatannotation(self.return_annotation)
-            rendered += " -> {}".format(anno)
         # ✂ end ✂
 
-        if len(rendered) > 70:
+        return result
+
+    def _return_annotation_str(self) -> str:
+        if self.return_annotation is not empty:
+            return formatannotation(self.return_annotation)
+        else:
+            return ""
+
+    def __str__(self):
+        result = self._params()
+        return_annot = self._return_annotation_str()
+
+        total_len = sum(len(x) + 2 for x in result) + len(return_annot)
+
+        if total_len > self.MULTILINE_CUTOFF:
             rendered = "(\n    " + ",\n    ".join(result) + "\n)"
-            if self.return_annotation is not _empty:
-                rendered += f" -> {formatannotation(self.return_annotation)}"
+        else:
+            rendered = "({})".format(", ".join(result))
+        if return_annot:
+            rendered += f" -> {return_annot}"
 
         return rendered
 
